# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mongodb_dataset']

package_data = \
{'': ['*']}

install_requires = \
['dnspython>=1.16.0,<2.0.0', 'pymongo>=3.10.1,<4.0.0']

setup_kwargs = {
    'name': 'mongodb-dataset',
    'version': '0.1.5',
    'description': 'Pythonic wrapper around pymongo that can act as a drop-in replacement for dataset',
    'long_description': '## mongodb-dataset\n\n![Build](https://github.com/TadpoleTutoring/mongodb-dataset/workflows/Python%20testing%20and%20linting/badge.svg)\n[![codecov](https://codecov.io/gh/TadpoleTutoring/mongodb-dataset/branch/master/graph/badge.svg?token=kh26hWszR0)](https://codecov.io/gh/TadpoleTutoring/mongodb-dataset)\n\n\nPythonic wrapper around pymongo that can act as a drop-in replacement for dataset \n',
    'author': 'InnovativeInventor',
    'author_email': 'root@max.fan',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/TadpoleTutoring/mongodb-dataset',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
